"""Django CMS Template Tags

.. moduleauthor:: Ryan Bagwell <ryan@ryanbagwell.com>


"""
from classytags.arguments import Argument
from classytags.core import Tag, Options
from django import template
from cms.models.titlemodels import Title

register = template.Library()


class PageURLBySlug(Tag):
    """ Looks up a page url by its slug.
        Why doesn't django CMS have this tag built in?
    """
    name = "page_url_by_slug"

    options = Options(
        Argument('page_slug', required=False, default=''),
    )

    def render_tag(self, context, page_slug):


        try:
            return Title.objects.filter(slug=page_slug).exclude(
                title='')[0].page.get_absolute_url()
        except:
            return ''

register.tag(PageURLBySlug)


import re
import os
from django.http import HttpResponsePermanentRedirect
from django.utils.html import strip_spaces_between_tags as short
from django.conf import settings


class SpacelessMiddleware(object):
    """ Removes spaces and line breaks
        from templates when DEBUG is False """

    def process_response(self, request, response):

        if not settings.DEBUG and 'text/html' in response['Content-Type']:
            response.content = short(response.content)

            response.content = '\n'.join([line for line in response.content.split('\n') if line.strip()])

        return response



class LegacyURLRedirectMiddleware:

    """ This middleware lets you match a specific url and redirect
        the request to an new url.

        You keep a tuple of url regex pattern/url redirect tuples on your site
        settings.

        For example, in settings.py:

        LEGACY_URL_REDIRECTS = (
            (r'^/about/news-events\.php(.+)?', '/news-events/$1',),
            (r'^/about/(?!index\.php)', '/about/',),
        )

        """

    def process_request(self, request):

        path = request.META['PATH_INFO']

        LEGACY_URL_REDIRECTS = getattr(settings, 'LEGACY_URL_REDIRECTS', None)

        if LEGACY_URL_REDIRECTS is None: return

        for url_pattern, redirect_url in LEGACY_URL_REDIRECTS:
            regex = re.compile(url_pattern)
            matched = regex.match(path)

            if matched:
                print matched.groups()

                for i, group in enumerate(matched.groups(), start=1):
                    print group
                    redirect_url = redirect_url.replace("$%s" % str(i), group)

                return HttpResponsePermanentRedirect(redirect_url)


from django.conf import settings
from django.core.management.base import LabelCommand
from django.contrib.redirects.models import Redirect
import optparse
import csv
import os
from urlparse import urlparse
from django.contrib.sites.models import Site



# A near-straight copy of
# github:richleland/django-cumulus/blob/27a595a/cumulus/management/commands/syncstatic.py
class Command(LabelCommand):

    help = ("Uploads a csv file of url redirects")

    option_list = LabelCommand.option_list + (
        optparse.make_option('-t', '--test-run',
            action='store_true', dest='test_run', default=False,
            help="Performs a test run of the upload."),
    )


    def handle_label(self, csv_file, **options):

        site = self._select_site()

        csv_file = os.path.abspath(os.path.expanduser(csv_file))

        csv_file = open(csv_file)

        reader = csv.reader(csv_file, delimiter=',', quotechar='"')

        for row in reader:

            if len(row) is not 2:
                print "Bad Redirect: %s" % row
                continue

            try:
                old_path = urlparse(row[0]).path
            except:
                print "Couldn't parse %s" % row[0]
                continue

            redirect, created = Redirect.objects.get_or_create(site=site,
                                                                old_path=old_path)

            redirect.new_path = row[1]

            redirect.save()



    def _select_site(self):
        sites = Site.objects.all()

        if sites.count() == 1:
            pk = sites[0].id
        else:
            print "Choose a site:"
            for s in site:
                print "%s - %s" % (site.pk, site.name)

            pk = raw_input("Your choice:")

        return Site.objects.get(pk=int(pk))




from setuptools import setup, find_packages

setup(
    name='django-muckraking',
    version='0.3.2',
    author='Ryan Bagwell',
    author_email='ryan@ryanbagwell.com',
    url='https://github.com/ryanbagwell/django-muckraking',
    packages=find_packages(),
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    description="A collection of random Django utilities",
    long_description=open('README.rst').read(),
    install_requires=[
        "Django >= 1.4",
        "django-classy-tags==0.4",
        "gitpython==0.3.2.RC1"
    ]
)



"""General-use emplate tags

.. moduleauthor:: Ryan Bagwell <ryan@ryanbagwell.com>


"""
from classytags.arguments import Argument
from classytags.core import Tag, Options
from django.conf import settings
from git import Repo
import os
from django import template

register = template.Library()


class EnqueueScript(Tag):
    """A WordPress-inspired template tag that allows you to add javascript to the scripts queue

    Args:
        name (str): a name for your script (i.e. jquery)
        src (str): the script's src attribute
        deps (str): a comma-seperated list of names of scripts that the script depends on
        version (int): the script version.

    """

    name = 'enqueue_script'

    options = Options(
        Argument('name', required=True),
        Argument('src', required=True),
        Argument('deps', required=False, default=''),
        Argument('version', required=False, default=1),
    )

    def render_tag(self, context, name, src, deps, version):
        queue = context.get('queue', {})
        queue[name]= DataScript(name, src, deps, version)
        context['queue'] = queue
        return ""

register.tag(EnqueueScript)



class RenderScripts(Tag):
    """A template tag that renders the scripts added to the script queue with {% enqueue_script %}

    """

    name = 'render_scripts'
    order = []

    def render_tag(self, context):
        self.queue = context.get('queue', {})

        for name, script in self.queue.iteritems():

            for dep in script.deps:

                if dep == '': continue

                if dep not in self.order:
                    self.order.append(dep)

                elif name in self.order:
                    popped = self.order.pop(self.order.index(name))
                    self.order.insert(self.order.index(dep) + 1, popped)

                else:
                    self.order.insert(self.order.index(dep) + 1, script.name)

            if name not in self.order:
                self.order.append(script.name)

        return '\n'.join([ self.queue[script].__unicode__() for script in self.order])

register.tag(RenderScripts)


class GitHeadHash(Tag):
    """ A template tag that outputs the HEAD commit hash
        of the current branch, if it exists.

        Useful for cache-busting url params """


    name = 'git_commit_hash'

    def render_tag(self, context):

        repo = Repo(os.path.dirname(__import__(settings.ROOT_URLCONF).__file__))

        return repo.heads[0].commit.hexsha

register.tag(GitHeadHash)

"""General-use emplate tags

.. moduleauthor:: Ryan Bagwell <ryan@ryanbagwell.com>


"""
from classytags.arguments import Argument
from classytags.core import Tag, Options
from django.conf import settings
from git import Repo
import os
from django import template
import hashlib

register = template.Library()


class EnqueueScript(Tag):
    """A WordPress-inspired template tag that allows you to add javascript to the scripts queue

    Args:
        name (str): a name for your script (i.e. jquery)
        src (str): the script's src attribute
        deps (str): a comma-seperated list of names of scripts that the script depends on
        version (int): the script version.

    """

    name = 'enqueue_script'

    options = Options(
        Argument('name', required=True),
        Argument('src', required=True),
        Argument('deps', required=False, default=''),
        Argument('version', required=False, default=1),
    )

    def render_tag(self, context, name, src, deps, version):
        queue = context.get('queue', {})
        queue[name]= DataScript(name, src, deps, version)
        context['queue'] = queue
        return ""

register.tag(EnqueueScript)



class RenderScripts(Tag):
    """A template tag that renders the scripts added to the script queue with {% enqueue_script %}

    """

    name = 'render_scripts'
    order = []

    def render_tag(self, context):
        self.queue = context.get('queue', {})

        for name, script in self.queue.iteritems():

            for dep in script.deps:

                if dep == '': continue

                if dep not in self.order:
                    self.order.append(dep)

                elif name in self.order:
                    popped = self.order.pop(self.order.index(name))
                    self.order.insert(self.order.index(dep) + 1, popped)

                else:
                    self.order.insert(self.order.index(dep) + 1, script.name)

            if name not in self.order:
                self.order.append(script.name)

        return '\n'.join([ self.queue[script].__unicode__() for script in self.order])

register.tag(RenderScripts)


class VersionHashTag(Tag):
    """ A template tag that outputs a unique hash
        for version purposes. First, it tries to obtain the
        latest git commit hash. If unsuccessful, it provides a hash
        of the last time the settings file was accessed.

        Useful for cache-busting url params """

    name = 'version_hash'

    def render_tag(self, context):

        git_hash = self.get_git_hash()

        if git_hash is '000000':
            return self.get_atime_hash()
        else:
            return git_hash

    def get_atime_hash(self):

        try:
            settings_file = __import__(settings.SETTINGS_MODULE).__file__
            a_time = os.stat(settings_file).st_atime
            h = hashlib.md5()
            h.update(str(a_time))
            return h.hexdigest()
        except:
            return '000000'

    def get_git_hash(self):

        try:
            repo = Repo(os.path.dirname(__import__(settings.ROOT_URLCONF).__file__))
            return repo.heads[0].commit.hexsha
        except:
            return '000000'

register.tag(VersionHashTag)


class GitHeadHash(VersionHashTag):
    """ A template tag that outputs the HEAD commit hash
        of the current branch, if it exists.

        Useful for cache-busting url params """

    name = 'git_commit_hash'

    def render_tag(self, context):

        return self.get_git_hash()

register.tag(GitHeadHash)


class SettingsFileTimeHash(VersionHashTag):
    """ A template tag that prints an md5 hash
        of the last time the Django settings file
        was accessed """

    name = 'settings_file_time_hash'

    def render_tag(self, context):

        return self.get_atime_hash()


register.tag(SettingsFileTimeHash)
